; Bits

kfLeft		equ 0001h		; Flag bit
kfRight		equ 0002h		;
kfUp		equ 0004h		;
kfDown		equ 0008h		;
kfFire		equ 0010h		;
kfSmart		equ 0020h		;
kfEsc		equ 0040h		;
kfOver		equ 003Fh		; These override replay

; Key scanning routines

; Start the key scanning

Init_Keys:	test b cs:Key_Replay_F,-1; 0 = record
		jnz Start_Key_Play	; Playing

		jmp Start_Key_Rec	; Recording

; Start the recording process

Start_Key_Rec:	mov b cs:Key_Replay_F,0	; Not replaying
		mov b cs:Key_Record_F,1	; Flag recording

		; Setup the rest

		push ds			; Save

		mov ax,DATA		; Segment
		mov ds,ax		;

		mov di,o Key_Data	; Rewind the ptr

		; Kill the buffer

		push di			; Save the ptr

		mov cx,Key_Data_E-Key_Data ; This much RAM

		mov al,0		; Clear to 00

		push es			; Save

		push ds			; In cs
		pop es			;

		rep stosb		; Clear it all

		pop es			; Restore

		pop di			; Restore ptr

		; Save various data

		mov w ax,cs:Seed	; Save the random seed
		mov ds:[di],ax		;
		mov w ax,cs:Seed+2	;
		mov ds:2[di],ax		;

		mov ax,cs:Score		; The score
		mov ds:4[di],ax		;

		mov al,cs:Shield_D	; The damage
		mov ds:6[di],al		;

		mov al,cs:Sheet_No	; The sheet number
		mov ds:7[di],al		;

		mov ax,cs:nMen		; The number of men
		mov ds:8[di],ax		;

		mov ax,cs:nMen_Coll	; The number of men collected
		mov ds:10[di],ax	;

		lea di,12[di]		; Skip

		mov ds:pKey_Data,di	; Save the ptr

		; Clear the count

		mov w ds:Key_Counts,0	; None yet

		; Done

		pop ds			; Restore
		ret			;

; Start the replay process

Start_Key_Play:	mov b cs:Key_Replay_F,1	; Replaying

		; Setup the rest

		push ds			; Save

		mov ax,DATA		; Segment
		mov ds,ax		;

		mov di,o Key_Data	; Rewind the ptr

		; Restore the random seed, etc

		mov ax,ds:[di]		; Restore the Rand seed
		mov cs:Seed,ax		;
		mov ax,ds:2[di]		;
		mov cs:Seed+2,ax	;

		mov ax,ds:4[di]		; The score
		mov cs:Score,ax		;

		mov al,ds:6[di]		; Damage
		mov cs:Shield_D,al	;

		mov al,ds:7[di]		; Sheet number
		mov cs:Sheet_No,al	;

		mov ax,ds:8[di]		; The number of men
		mov cs:nMen,ax		;

		mov ax,ds:10[di]	; The number of men collected
		mov cs:nMen_Coll,ax	;

		lea di,12[di]		; Skip

		; Clear the keystate

		mov w cs:Key_Flags,0	; None yet

		mov ax,ds:[di]		; Get the count
		mov ds:Key_Counts,ax	; This long before next

		mov ds:pKey_Data,di	; Store the ptr

		; Done

		pop ds			; Restore
		ret			;

	; Stop replaying the list

GKSR_Stop:	mov b cs:Key_Replay_F,0	; Stop the replay
		mov b cs:Key_Record_F,1	; Flag record keys

		; Now, we must patch-up the recording so we can add to it

		push ds			; Save
		push ax			;

		mov bx,ax		; Save keys

		mov ax,DATA		; Segment
		mov ds,ax		;

		mov di,ds:pKey_Data	; Point at the data

		; Now, calc the count value

		mov ax,ds:[di]		; Get the last count
		sub ax,ds:Key_Counts	; Calc how many elapsed
		inc ax			; Allow for the fact that at least
					; one has elapsed

		; This is how long we were in this state

		mov ds:[di],ax		; Write new count
		mov ds:2[di],bx		; Save new key-state

		; Move the ptr

		lea di,4[di]		; Skip this one

		mov ds:pKey_Data,di	; Save the ptr

		; Setup for next state

		mov cs:Key_Flags,bx	; This is now the current state
		mov w ds:Key_Counts,0	; Zero the count

		; Return this key state, continue

		mov ax,bx		; Get it		

		pop ax			; Restore
		pop ds			;

		ret			; Done

; Replay a key-list

GKS_Replay:	test ax,kfOver		; Overridden ?
		jnz GKSR_Stop		; Yes, stop replaying

		push ds			; Save

		mov ax,DATA		; Segment
		mov ds,ax		;

		dec w ds:Key_Counts	; Next state yet ?
		jnz GKSR_X		; No, skip

		; We need the next state

		mov di,ds:pKey_Data	; Point at the data

		mov ax,ds:2[di]		; Flags
		mov cs:Key_Flags,ax	; Set them

		lea di,4[di]		; Skip this data

		mov ds:pKey_Data,di	; Save the new ptr

		; Get the next count

		mov ax,ds:[di]		; Count
		mov ds:Key_Counts,ax	;

		; Test the escape key

GKSR_X:		test b cs:Key_Matrix+1,1; Esc ?
		jz GKSR_NoEsc		; No, skip

		mov ax,kfEsc		; Set it		
		or cs:Key_Flags,ax	;

		; Hack the smart-bomb key

GKSR_NoEsc:	pop ds			; Restore
		ret			; Done

; Add an escape key to the list

End_Keys:	test b cs:Key_Replay_F,1; Are we replaying the list ?
		jne EK_X		; Yes, skip

		; Record it

		mov ax,kfEsc		; ESC pressed
		jmp s GKS_NoEsc		; Dummy state

EK_X:		ret			; Done

; Get the key-states into Key_Flags for the game

Get_Key_States:	mov ax,0		; None pressed

		; Scan the direction keys

		mov di,o Key_Left	; Left ?
		mov cx,kfLeft		; Flag

		call GKS_UDK		; Drive it

		mov di,o Key_Right	;
		mov cx,kfRight		;

		call GKS_UDK		;

		mov di,o Key_Up		;
		mov cx,kfUp		;

		call GKS_UDK		;

		mov di,o Key_Down	;
		mov cx,kfDown		;

		call GKS_UDK		;

		mov di,o Key_Fire	;
		mov cx,kfFire		;

		call GKS_UDK		;

		mov di,o Key_Smart	;
		mov cx,kfSmart		;

		call GKS_UDK		;

		; Test the escape key

		test b cs:Key_Matrix+1,1; Esc ?
		jz GKS_NoEsc		; No, skip

		or ax,kfEsc		; Set it		

		; See if replay

GKS_NoEsc:	test b cs:Key_Replay_F,1; Are we replaying the list ?
		jne vGKS_Replay		; Yes, skip

		; No, are we recording

		push ds			; Save 

		mov bx,DATA		; Segment
		mov ds,bx		;

		; Step the timer

		inc ds:Key_Counts	; One more frame

		; Is this a new state ?

		cmp cs:Key_Flags,ax	; Is this a new state ?
		je GKS_Exit		; No, skip

		; Save the next state

		mov cs:Key_Flags,ax	; Save the new keys

		; Are we recording ?

		test b cs:Key_Record_F,1; Are we recording the list ?
		jz GKS_Exit		; No, skip

		; Point at the key-data list

		mov di,ds:pKey_Data	; Get the pointer

		mov ds:2[di],ax		; State

		mov ax,ds:Key_Counts	; Number of frames
		mov ds:[di],ax		; Count

		lea di,4[di]		; Move on

		mov ds:pKey_Data,di	; Store the new ptr

		; Clear the counter

		mov w ds:Key_Counts,0	; Back to 0

		; Done

GKS_Exit:	pop ds			; Restore
GKS_End:	ret			;

vGKS_Replay:	jmp GKS_Replay		; Goddam short branches

	; Test a key and set the bit if pressed

GKS_UDK:	mov bl,cs:[di]		; Get the next key-code

		cmp bl,0		; End ?
		je GKSU_X		; Yes, skip (Z)

		mov bh,0		; -16 bit
		add bx,o Key_Matrix	; Add base

		test b cs:[bx],1	; Pressed ?
		jne GKSU_Pressed	; Yes, skip NZ

		inc di			; Skip to the next key

		jmp s GKS_UDK		; Loop

	; Pressed

GKSU_Pressed:	or ax,cx		; Set the flag

	; Exit

GKSU_X:		ret			; Done

	; Test a key. NZ = pressed

Test_UDK:	mov bl,cs:[di]		; Get the next key-code

		cmp bl,0		; End ?
		je GKSU_X		; Yes, skip (Z)

		mov bh,0		; -16 bit
		add bx,o Key_Matrix	; Add base

		test b cs:[bx],1	; Pressed ?
		jne GKSU_X		; Yes, skip NZ

		inc di			; Skip to the next key

		jmp s Test_UDK		; Loop

; Save bx

Test_UDK_S:	push bx			; Save
		call Test_UDK		; Test it
		pop bx			; Restore

		ret			; Done

; The far key tests for tetris

Far_Test_Left:	mov di,o Key_Left	; Left ?
		call Test_UDK_S		; Drive it

		retf			; Done

Far_Test_Right:	mov di,o Key_Right	;
		call Test_UDK_S		;

		retf			;

Far_Test_RotL:	mov di,o Key_Up		;
		call Test_UDK_S		;

		retf			;

Far_Test_RotR:	mov di,o Key_Down	;
		call Test_UDK_S		;

		retf			;

Far_Test_Drop:	mov di,o Key_Fire	;
		call Test_UDK_S		;

		retf			;

Far_Test_Pause:	mov di,o Key_Pause	;
		call Test_UDK_S		;

		retf			;

Far_Test_Esc:	mov di,o Key_Esc	;
		call Test_UDK_S		;

		retf			;

; The current key data

Key_Up		db kN,kUp		; The Up-keyset
		db UDK_nKeys-1 dup (0)	;

Key_Down	db kM,kDown		;
		db UDK_nKeys-1 dup (0)	;

Key_Left	db kZ,kLeft		;
		db UDK_nKeys-1 dup (0)	;

Key_Right	db kX,kRight		;
		db UDK_nKeys-1 dup (0)	;

Key_Fire	db kSPACE		;
		db UDK_nKeys dup (0)	;

Key_Smart	db kTAB,kQ,kW,kE,kR	;
		db UDK_nKeys-4 dup (0)	;

Key_Pause	db kSCROLL		;
		db UDK_nKeys dup (0)	;

Key_Boss	db kF5,kF6,kF7,kF8	;
		db UDK_nKeys-3 dup (0)	;

Key_Esc		db kEsc,0		; Dummy escape key

; Vars

		align 2

Key_Flags	dw ?			; The state of the key-board

Key_Hack	db 0			; Hack
Key_Replay_F	db 0			; NZ = replay list
Key_Record_F	db 0			; NZ = recording list
