; Drive the game top of screen displays

Init_Display:	call Draw_Radar		; Show it

		; And the other page

		call Swap_Pages		; Flip them

		call Draw_Radar		; Again

		; Setup the shields

		mov b cs:Shield_A,0	; Start at 0
		mov b cs:Shield_Flag,0	; Clear the flag

		; Setup the score / clock

		mov si,o Disp_Init	; Initial data
		mov di,o Disp_Data	; Vars
		mov cx,8 * (6+5+1)	; Length

		call LDIR		; Copy it

		; Start the clock

		mov di,o Clock_Digits	; The digits

		mov cx,6		; Clear the time

ID_Lp:		mov b cs:[di],0		; Digit = 0
		inc di			;

		loop ID_Lp		; Loop

		; Setup the score

		mov ax,cs:Score		; Get the score

		call Scroll_Dec_16	; Setup to scroll it
		
		; Start a clock-scroll
	
Set_Up_Sec:	mov b cs:Sec_Count,60	; Frames per second

		; Should we step it ?

		test b cs:Action,1	; Over ?
		jz CAD_Done		; Yes, skip

		; Step the time by one second

		mov si,o Clock_Digits	; Current digits
		mov di,o Clock_Chars	; Variables

		mov ch,10		; First digit
		call CD_Digit		;
		jnz CAD_Done		; Done

		mov ch,6		;
		call CD_Digit		;
		jnz CAD_Done		; Done

		mov ch,10		;
		call CD_Digit		;
		jnz CAD_Done		; Done

		mov ch,6		;
		call CD_Digit		;
		jnz CAD_Done		; Done

		mov ch,10		;
		call CD_Digit		;
		jnz CAD_Done		; Done

		mov ch,10		;
		call CD_Digit		;

CAD_Done:	ret			; Done

CD_Digit:	mov al,cs:[si]		; Step digit
		inc al			;

		cmp al,ch		; Max count ?
		jc CDT_NotWrap		; No

		xor al,al		; Reset digit

		mov ch,0		; Flag that we wrapped 

CDT_NotWrap:	mov cs:[si],al		; Store the digit

		cbw			; 16-bit
		add ax,ax		; * 8
		add ax,ax		;
		add ax,ax		;

		add ax,o dSc0		; Ptr to the CS
		mov cs:4[di],ax		;
		sub ax,8		; Previous char
		mov cs:2[di],ax		;

		mov w cs:6[di],00008h	; Offset/count

		inc si			; Step to the next digit

		lea di,-8[di]		; Step to the next character

		or ch,ch		; Set Z/NZ for wrapped
		ret			; Done

; Write a number as a scrolling score

Scroll_Dec_16:	push ax			; Save regs
		push bx			;
		push cx			;
		push dx			;

		push di			;
		push si			;

		mov cx,0		; Flag that nothing scrolled

		mov cs:Curr_Score,ax	; Save the current number
		mov b cs:Score_Count,15	; Time

		mov di,ax		; Number

		mov si,o Score_Chars	; Point at the char data

		mov bx,o PowersOfTen	; Digit values table

		mov ah,-1		; To be displayed for 0

SCD_Lp:		call WD_CalcDigit	; Calc digit

		cmp al,0		; Zero ?
		jnz SCD_1		; No

		mov al,ah		; Print ' ' or '0'
		call Scroll_Dec_Char	;

		jmp s SCD_Next		; Do next digit

SCD_1:		mov ah,0		; Use 0 in future
		
		call Scroll_Dec_Char	; Add it

SCD_Next:	add bx,2		; Next power of ten

		cmp w cs:[bx],1		; Last power of ten ?
		jnz SCD_Lp		; No, continue

		call WD_CalcDigit	; Calc digit

		call Scroll_Dec_Char	;

		pop si			; Restore
		pop di			;
		pop dx			;
		pop cx			;
		pop bx			;
		pop ax			;

		ret			; Done

; Add this char

Scroll_Dec_Char:push ax			; Save

		cmp al,-1		; Blank ?
		je SDC_Blank		; Yes, skip

		cbw			; 16-bit

		add ax,ax		; * 8
		add ax,ax		;
		add ax,ax		;

		add ax,o dSc0		; 0

SDC_1:		mov dx,cs:4[si]		; New cs
		mov cs:2[si],dx		; Use as old

		mov cs:4[si],ax		; New

		cmp ax,dx		; Same as old ?
		je SDC_2		; Yes, don't scroll it

		mov cx,00008h		; Set the counts

SDC_2:		cmp cx,0		; Count = 0 ?
		jz SDC_3		; Yes, skip

		mov cs:6[si],cx		; Counts

SDC_3:		lea si,8[si]		; Skip this char

		pop ax			; Restore
		ret			; Done

SDC_Blank:	mov ax,o dScS		; Blank
		jmp s SDC_1		; As above

; Damage the shield by al.

Shield_Dec:	mov ah,cs:Difficulty	; Get the degree of difficulty
		and ah,03h		; Mask/ = 0  ?
		je SD_Train		; Yes, skip

		cmp ah,3		; Hard
		jne SD_Dam		; No, damage it

		add al,al		; Damage * 4
		add al,al		;

	; Do damage

SD_Dam:		sub b cs:Shield_D,al	; Dec it
		jc SD_Clip		; Clip it to 00

SD_X:		ret			; Done

	; Damage := 0

SD_Clip:	mov b cs:Shield_D,0	; Kill it

		ret			; Done
 
	; Only damage if the ship is firing

SD_Train:	test w cs:Key_Flags,kfFire ; Fire ?
		jnz SD_Dam		; Yes, damage it

		ret			; Done

; Repair the shield by al.

Shield_Inc:	add al,cs:Shield_D	; Inc it
		cmp al,63		; Clip to this
		jbe SI_Ok		; Clip it to 63

		mov al,63		; Clip it

SI_Ok:		mov cs:Shield_D,al	; Save it
		ret			; Done

SI_Clip:	mov b cs:Shield_D,63	; Clip it

		ret			; Done
 
; Step the clock, etc

Show_Display:	mov ax,cs:VDU_Seg	; Set it to point at VDU
		mov ds,ax		;

		call Set_EGA_WM0	; Setup for update

		; Drive the shields

		mov al,cs:DRAW_Page	; Odd or even ?
		cmp al,0		;
		jnz vSD_S_Dup		; Odd, skip

		; It is the even field, do we need to change the bar-graph ?

		mov al,cs:Shield_A	; Actual shields
		cmp al,cs:Shield_D	; Demand shields
		jz vSD_NotMove		; Same, skip

		mov ah,1		; Assume +ve
		jb SD_S_1		; Move up

		mov ah,-1		; -ve

SD_S_1:		add al,ah		; New actual
		mov cs:Shield_A,al	; Set it

		; Now, we need to draw the bar-graph

		mov ch,al		; Save the width

		mov cl,al		; Bottom bits
		and cl,07h		;

		shr al,1		; /8
		shr al,1		;
		shr al,1		;

		cbw			; 16-bit

		mov bx,ax		; bx = pRam
		add bx,cs:VDU_Width	; Offset + 2 lines
		add bx,cs:VDU_Width	;
		add bx,23		; + 23 chars
		add bx,cs:VDU_Offset	;

		mov al,80h		; Bit 
		sar al,cl		;

		; Is the bar-graph 0 ?

		cmp ch,0		; Are we dead ?
		jnz SD_S_Alive		; No, skip

		mov al,00		; Clear the mask

		; Save the ptr to the bar-graph

SD_S_Alive:	mov cs:Shield_Ptr,bx	; Save the ptr
		mov cs:Shield_Msk,al	; Save the mask

		mov b cs:Shield_Flag,1	; Flag draw operation

		; Draw the bar-graph

		call Draw_Bar_Graph	; Draw it

		; Done, skip

		jmp s SD_DoScore	; Skip

; Vectors

vSD_NotMove:	jmp s SD_NotMove	; Vector
vSD_S_Dup:	jmp s SD_S_Dup		;

	; Draw the bar-graph

Draw_Bar_Graph:	mov [bx],al		; Write the byte
		add bx,cs:VDU_Width	; Down
		mov [bx],al		;
		sub bx,cs:VDU_Width	; Up

		; Now, is this a #FF ?

		cmp al,0FFh		; A full byte
		je DBG_1		; Yes, skip

		; No, done

		ret			; Done

	; Write the next byte

DBG_1:		mov b 1[bx],0		; Clear it
		add bx,cs:VDU_Width	; Down
		mov b 1[bx],0		;
		sub bx,cs:VDU_Width	; Up

		ret			; Done

	; Draw the second frame

SD_S_Dup:	test b cs:Shield_Flag,-1; Should we draw it
		jz SD_DoScore		; No, skip

		mov bx,cs:Shield_Ptr	; Get the ptr
		mov al,cs:Shield_Msk	; Get the mask

		; Draw it

		call Draw_Bar_Graph	; Draw the bar

		; Are we dead ?

		cmp al,0		; End ?
		jnz SD_NotMove		; No, skip

		; Shields are dead, flag dead ship

		mov b cs:Game_Flag,3	; Flag dead

	; Clear the flag

SD_NotMove:	mov b cs:Shield_Flag,0	; No change

	; Drive the score

SD_DoScore:	mov al,cs:Score_Count	; Get the score count
		cmp al,0		; Scrolling ?
		jz SD_0			; No, skip

		dec al			; Dec cnt
		mov cs:Score_Count,al	; Store it

		jmp s SD_Clock		; Skip

		; Have we got a new score ?

SD_0:		mov ax,cs:Score		; Get the score
		cmp cs:Curr_Score,ax	; Same as shown ?
		je SD_Clock		; Yes, skip

		; Setup for scroll

		call Scroll_Dec_16	; Scroll the score

		; Drive the clock

SD_Clock:	mov al,cs:Sec_Count	; Dec clock cnt
		dec al			;
		mov cs:Sec_Count,al	;

		; Tick or colon on

		or al,al		; Zero ?
		je SD_Tick		; Yes, step the clock

		and al,0FEh		; 0/1 ?
		je SD_Colon		; Yes, set the colons

		cmp al,30		; 30/31 ?
		jne SD_Update		; No, skip

		; Clear the colons

		mov al,0		; Kill them
		call Draw_Colons	;

		jmp s SD_Update		; Skip

	; Step the clock

SD_Tick:	call Set_Up_Sec		; Step one second

		; Show the colons

SD_Colon:	mov al,018h		; Shape
		call Draw_Colons	;

		; Now, see if any digits need scrolling

SD_Update:	mov si,o Disp_Data	; Vars

		mov bx,cs:[si]		; Get the display ptr
		add bx,cs:VDU_Offset	;
		
		mov al,cs:DRAW_Page	; Odd or even ?
		cmp al,0		;
		jnz SDU_O_1		; Odd, skip

		; This writes the new characters

		jmp s SDU_E_1		; Start it

	; Next char

SDU_E_Lp:	lea si,8[si]		; Next

		mov bx,cs:[si]		; Get the display ptr
		cmp bx,-1		; End ?
		je SDU_X		; Yes, done

		add bx,cs:VDU_Offset	; Offset it

		; Drive the next char

SDU_E_1:	mov cx,cs:6[si]		; Counts
		cmp cx,0		; End ?
		jz SDU_E_Lp		; Yes, skip

		; Any top half ?

		cmp cl,0		; Any ?
		je SDU_E_Bott		; No, show the bottom half

		; Get the ptr

		mov di,cs:2[si]		; Ptr to top half CS

		; Show the old char (top)

SDU_E_Lp1:	mov ah,cs:[di]		; Get 8 pixels
		inc di			; Step the ptr

		and b [bx],ah		; Write the pixels
		add bx,cs:VDU_Width	; Next line

		dec cl			; Loop
		jnz SDU_E_Lp1		;

		; Now, show the bottom half, is there one

SDU_E_Bott:	cmp ch,0		; Any ?
		je SDU_E_Lp		; No, done

		; Get the ptr

		mov di,cs:4[si]		; Ptr to bottom half CS

		; Show the new char (bottom)

SDU_E_Lp2:	mov ah,cs:[di]		; Get 8 pixels
		inc di			; Step the ptr

		and b [bx],ah		; Write the pixels
		add bx,cs:VDU_Width	; Next line

		dec ch			; Loop
		jnz SDU_E_Lp2		;

		; Now done, next

		jmp s SDU_E_Lp		; Loop

SDU_X:		ret			; Done

	; Kill this char

SDU_O_Kill:	mov w cs:6[si],0	; Clear the counts

	; Next char, ODD

SDU_O_Lp:	lea si,8[si]		; Next

		mov bx,cs:[si]		; Get the display ptr
		cmp bx,-1		; End ?
		je SDU_X		; Yes, done

		add bx,cs:VDU_Offset	; Offset

		; Drive the next char

SDU_O_1:	mov cx,cs:6[si]		; Counts
		cmp cx,0		; End ?
		jz SDU_O_Lp		; Yes, skip

		; Any top half ?

		cmp cl,0		; Any ?
		je SDU_O_Bott		; No, show the bottom half

		; Show the old char (top)

		mov di,cs:2[si]		; Point at the char

SDU_O_Lp1:	mov ah,cs:[di]		; Get 8 pixels
		inc di			; Step the ptr

		and b [bx],ah		; Write the pixels
		add bx,cs:VDU_Width	; Next line

		dec cl			; Loop
		jnz SDU_O_Lp1		;

		; Now, show the bottom half, is there one ?

SDU_O_Bott:	cmp ch,0		; Any ?
		je SDU_O_NB		; No, skip

		mov di,cs:4[si]		; Ptr to bottom half CS

		; Show the new char (bottom)

SDU_O_Lp2:	mov ah,cs:[di]		; Get 8 pixels
		inc di			; Step the ptr

		and b [bx],ah		; Write the pixels
		add bx,cs:VDU_Width	; Next line

		dec ch			; Loop
		jnz SDU_O_Lp2		;

		; Now, step the counts, if we have not finished

SDU_O_NB:	mov ax,cs:6[si]		; Get counts

		cmp al,0		; Top = 0 ?
		je SDU_O_Kill		; Yes, kill this char

		; Move ptrs

		inc w cs:2[si]		; pTop

		; Dec top, inc bott

		dec al			; One less
		inc ah			; One more

		mov cs:6[si],ax		; Store them

		; Now done, next

		jmp s SDU_O_Lp		; Loop

; Set the EGA/VGA registers for a write mode 0 operation

Set_EGA_WM0:	; Set Write mode 0 (rm 1)

		mov dx,3CEh		; Graphics controller

		mov ax,00805h		; Mode/Reg
		out dx,ax		; Set it

		; Set the colour don't care register

		mov ax,00007h		; CDC := 00
		out dx,ax		;

		; Set the function to replace

		mov ax,00003h		; DRF := 0
		out dx,ax		;

		; Enable all the bits

		mov ax,0FF08h		; BIT MASK = FF
		out dx,ax		;

		; Disable the set-reset register

		mov ax,00001h		; Disable SET/RESET
		out dx,ax		;

		; Done, end

		ret			; Done

; Copy from cs:[si] to cs:[di] for length cx

LDIR:		mov al,cs:[si]		; Read
		mov cs:[di],al		; Write

		inc si			; Step
		inc di			;

		loop LDIR		; Loop

		ret			; Done

; Draw/Hide the colons

Draw_Colons:	mov bx,34 		; Screen address of LHS colon
		add bx,cs:VDU_Width	; Next line
		add bx,cs:VDU_Offset	; Offset

		call DC_1		; Colon

		lea bx,3[bx]		; Point at the other colon

		; Show a colon

DC_1:		push bx			; Save

		mov [bx],al		; Show it
		add bx,cs:VDU_Width	;
		mov [bx],al		;

		add bx,cs:VDU_Width	;
		add bx,cs:VDU_Width	;

		mov [bx],al		;
		add bx,cs:VDU_Width	;
		mov [bx],al		;

		pop bx			; Restore

		ret			; Done

; Fixed data

Disp_Init	dw 32,dSc9,dSc0		; Clock LHS
		db 8,0			;

		dw 33,dScS,dSc0		;
		db 8,0			;

		dw 35,dScS,dSc0		;
		db 8,0			;

		dw 36,dScS,dSc0		;
		db 8,0			;

		dw 38,dScS,dSc0		;
		db 8,0			;

		dw 39,dScS,dSc0		; Clock RHS
		db 8,0			;

		; Data for the score

		dw 1,dScS,dScS		; Score LHS (x=0, really)
		db 0,0			;

		dw 2,dScS,dScS		;
		db 0,0			;

		dw 3,dScS,dScS		;
		db 0,0			;

		dw 4,dScS,dScS		;
		db 0,0			;

		dw 5,dScS,dSc0		;
		db 8,0			;

		dw -1,0,0,0		; Term

		; Data ends

		db 03Ch,066h,066h,03Eh ; Copy of 9
		db 006h,066h,03Ch,000h

dSc0		db 03Ch,066h,066h,066h
		db 066h,066h,03Ch,000h

dSc1		db 018h,038h,018h,018h
		db 018h,018h,07Eh,000h

dSc2		db 03Ch,066h,006h,03Ch
		db 060h,066h,07Eh,000h

dSc3		db 03Ch,066h,006h,01Ch
		db 006h,066h,03Ch,000h

dSc4		db 01Ch,03Ch,06Ch,0CCh
		db 0FEh,00Ch,01Eh,000h

dSc5		db 07Eh,062h,060h,07Ch
		db 006h,066h,03Ch,000h

dSc6		db 03Ch,066h,060h,07Ch
		db 066h,066h,03Ch,000h

dSc7		db 07Eh,066h,006h,00Ch
		db 018h,018h,018h,000h

dSc8		db 03Ch,066h,066h,03Ch
		db 066h,066h,03Ch,000h

dSc9		db 03Ch,066h,066h,03Eh
		db 006h,066h,03Ch,000h

		db 03Ch,066h,066h,066h	; Copy of 0
		db 066h,066H,03Ch,000h

dScS		db 8 dup (0)		; Blank

; Vars

Shield_A	db 0			; Actual pos
Shield_D	db 0			; Demand posn

Shield_Ptr	dw 0			; Ptr to the VDU
Shield_Msk	db 0			; Bit-mask
Shield_Flag	db 0			;

Sec_Count	db 0			; 60 .. 0
Score_Count	db 0			; 8 .. 0

Curr_Score	dw 0			; The current score

Clock_Digits	db 6 dup (0)		; Clock digits

Disp_Data	db 5 * 8 dup(0)		; Clock characters
Clock_Chars	db     8 dup(0)		; Last clock char

Score_Chars	db 5 * 8 dup(0)		; Score chars

		db 8 dup(0)		; Terminator
